/*********************************************************************
 *
 * $Id: yocto_api_html.ts 43403 2021-01-19 11:58:02Z mvuilleu $
 *
 * High-level programming interface, common to all modules
 *
 * - - - - - - - - - License information: - - - - - - - - -
 *
 *  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
 *
 *  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
 *  non-exclusive license to use, modify, copy and integrate http
 *  file into your software for the sole purpose of interfacing
 *  with Yoctopuce products.
 *
 *  You may reproduce and distribute copies of this file in
 *  source or object form, as long as the sole purpose of this
 *  code is to interface with Yoctopuce products. You must retain
 *  this notice in the distributed source file.
 *
 *  You should refer to Yoctopuce General Terms and Conditions
 *  for additional information regarding your rights and
 *  obligations.
 *
 *  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT
 *  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
 *  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
 *  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
 *  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
 *  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
 *  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
 *  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
 *  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
 *  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
 *  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
 *  WARRANTY, OR OTHERWISE.
 *
 *********************************************************************/

export * from "./yocto_api.js";

import {
    YUnhandledPromiseRejectionCallback,
    _YY_UrlInfo,
    YConditionalResult,
    YHTTPBody,
    YHTTPRequest,
    YErrorMsg,
    YSystemEnv,
    Y_YHubType,
    YGenericHub,
    YWebSocketHub,
    YGenericSSDPManager,
    YAPIContext,
    YAPI, YAPI_SUCCESS
} from "./yocto_api.js";

/**
 * System environment definition, for use in a browser
 */
export class YSystemEnvHtml extends YSystemEnv
{
    isNodeJS: boolean = false;
    hasSSDP: boolean = false;

    hookUnhandledRejection(handler: YUnhandledPromiseRejectionCallback)
    {
        window.addEventListener('onunhandledrejection', (event: Event) => {
            let promiseRejectionEvent = <PromiseRejectionEvent>event;
            handler(promiseRejectionEvent.reason, promiseRejectionEvent.promise);
        });
    }

    getWebSocketHub(obj_yapi: YAPIContext, urlInfo: _YY_UrlInfo): YGenericHub | null
    {
        return new YWebSocketHtmlHub(obj_yapi, urlInfo);
    }

    getHttpHub(obj_yapi: YAPIContext, urlInfo: _YY_UrlInfo): YGenericHub | null
    {
        return new YHttpHtmlHub(obj_yapi, urlInfo);
    }

    getWebSocketCallbackHub(obj_yapi: YAPIContext, urlInfo: _YY_UrlInfo, ws: any): YGenericHub | null
    {
        return obj_yapi._throw(YAPI.NOT_SUPPORTED, 'WebSocket Callback mode is not available in this environment', null);
    }

    getHttpCallbackHub(obj_yapi: YAPIContext, urlInfo: _YY_UrlInfo, incomingMessage: any, serverResponse: any): YGenericHub | null
    {
        return obj_yapi._throw(YAPI.NOT_SUPPORTED, 'HTTP Callback mode is not available in this environment', null);
    }

    getSSDPManager(obj_yapi: YAPIContext): YGenericSSDPManager | null
    {
        return obj_yapi._throw(YAPI.NOT_SUPPORTED, 'Hub discovery is not available in this environment', null);
    }

    loadfile(file: string | Blob): Promise<Uint8Array>
    {
        return new Promise((resolve: Function, reject: Function) => {
            let reader = new FileReader();
            reader.onerror = function (evt) {
                // @ts-ignore
                reject(evt.target.error);
            };
            reader.onload = function (evt) {
                // @ts-ignore
                resolve(new Uint8Array(evt.target.result));
            };
            reader.readAsArrayBuffer(<Blob>file);
        });
    }

    downloadfile(url: string): Promise<Uint8Array>
    {
        return new Promise((resolve, reject) => {
            let httpRequest = new XMLHttpRequest();
            httpRequest.open('GET', url, true);
            httpRequest.overrideMimeType('text/plain; charset=x-user-defined');
            httpRequest.onreadystatechange = (() => {
                if (httpRequest.readyState == 4) {
                    if(httpRequest.status != 200 && httpRequest.status != 304) {
                        if(httpRequest.status) {
                            reject(new Error('HTTP error ' + httpRequest.status));
                        } else {
                            reject(new Error('Unable to complete HTTP request, network down?'))
                        }
                    } else {
                        resolve(YAPI.imm_str2bin(httpRequest.responseText));
                    }
                }
            });
            httpRequest.send('');
        });
    }
}

const _HtmlSystemEnv: YSystemEnvHtml = new YSystemEnvHtml();

YAPI.imm_setSystemEnv(_HtmlSystemEnv);

class YHttpHtmlHub extends YGenericHub
{
    notbynRequest: XMLHttpRequest | null                  = null;
    notbynOpenPromise: Promise<YConditionalResult> | null = null;

    constructor(yapi: YAPIContext, urlInfo: _YY_UrlInfo)
    {
        super(yapi, urlInfo);
        this.notbynRequest     = null;
        this.notbynOpenPromise = null;
    }

    /** Handle HTTP-based event-monitoring work on a registered hub
     *
     * @param mstimeout {number}
     * @param errmsg {YErrorMsg}
     * @returns {number}
     */
    async testHub(mstimeout: number, errmsg: YErrorMsg): Promise<number>
    {
        if (this.disconnecting) {
            if(errmsg) {
                errmsg.msg = "I/O error";
            }
            return YAPI.IO_ERROR;
        }
        let args = '?len=' + this.notiflen.toString();
        if (this.notifPos > 0) {
            args += '&abs=' + this.notifPos.toString();
        }
        if(!this.notbynOpenPromise) {
            this.notbynOpenTimeout = (mstimeout ? this._yapi.GetTickCount() + mstimeout : null);
            this.notbynOpenPromise = new Promise(
                (resolve, reject) => {
                    this.notbynTryOpen = () => {
                        let xmlHttpRequest = new XMLHttpRequest();
                        this._firstArrivalCallback = true;
                        this.notbynRequest = xmlHttpRequest;
                        xmlHttpRequest.open('GET', this.urlInfo.url+'not.byn'+args, true, '', '');
                        xmlHttpRequest.overrideMimeType('text/plain; charset=x-user-defined');
                        xmlHttpRequest.onreadystatechange = (() => {
                            if (this.disconnecting) {
                                return;
                            }
                            if (xmlHttpRequest.readyState >= 3) {
                                if (xmlHttpRequest.readyState == 4 &&
                                    (xmlHttpRequest.status >> 0) != 200 &&
                                    (xmlHttpRequest.status >> 0) != 304) {
                                    // connection error
                                    if(!this.imm_testHubAgainLater()) {
                                        resolve({ errorType: YAPI.IO_ERROR, errorMsg: "I/O error" });
                                    }
                                } else {
                                    // receiving data properly
                                    if(!this._hubAdded) {
                                        this.signalHubConnected().then(() => {
                                            resolve({ errorType: YAPI_SUCCESS, errorMsg: "" });
                                        });
                                    }
                                    if (xmlHttpRequest.readyState == 3) {
                                        // when using reconnection mode, ignore state 3
                                        if (this.notiflen == 1) return;
                                    }
                                    let newlen = xmlHttpRequest.responseText.length;
                                    if (newlen > this.currPos) {
                                        this._yapi.parseEvents(this, xmlHttpRequest.responseText.substr(this.currPos, newlen - this.currPos));
                                    }
                                    // trigger immediately a new connection if closed in success
                                    if (xmlHttpRequest.readyState == 4 && (xmlHttpRequest.status >> 0) != 0) {
                                        this.notbynOpenPromise = null;
                                        this.currPos = 0;
                                        this.testHub(0, errmsg);
                                    }
                                }
                            }
                        });
                        this.notbynRequest.send('');
                    };
                    this.notbynTryOpen();
                }
            );
        }
        let res_struct = await this.notbynOpenPromise;
        if (errmsg) {
            errmsg.msg = res_struct.errorMsg;
        }
        this.notbynOpenPromise = null;
        return res_struct.errorType;
    }

    /** Perform an HTTP query on the hub
     *
     * @param method {string}
     * @param devUrl {string}
     * @param obj_body {YHTTPBody|null}
     * @param tcpchan {number}
     * @returns {YHTTPRequest}
     */
    async request(method: string, devUrl: string, obj_body: YHTTPBody | null, tcpchan: number): Promise<YHTTPRequest>
    {
        return new Promise<YHTTPRequest>(
            (resolve, reject) => {
                let prefix = this.urlInfo.url.slice(0,-1);
                let httpRequest = new XMLHttpRequest();
                httpRequest.open(method, prefix + devUrl, true, this.urlInfo.user, this.urlInfo.pass);
                httpRequest.onreadystatechange = (() => {
                    if (httpRequest.readyState == 4) {
                        let yreq = new YHTTPRequest(null);
                        if(httpRequest.status != 200 && httpRequest.status != 304) {
                            yreq.errorType = (httpRequest.status == 401 ? YAPI.UNAUTHORIZED : YAPI.NOT_SUPPORTED);
                            yreq.errorMsg = 'HTTP Error '+httpRequest.status+' on '+prefix+devUrl;
                        } else {
                            yreq.bin_result = this._yapi.imm_str2bin(httpRequest.responseText);
                        }
                        resolve(yreq);
                    }
                });
                let body: FormData | string = '';
                if(obj_body) {
                    let blob = new Blob([obj_body.data], {type: 'application/octet-binary'});
                    body = new FormData();
                    body.append(obj_body.fname, blob);
                }
                httpRequest.send(body || '');
            }
        );
    }

    async disconnect()
    {
        this.imm_commonDisconnect();
        if(this.notbynRequest) {
            (<XMLHttpRequest> this.notbynRequest).abort();
        }
    }
}

class YWebSocketHtmlHub extends YWebSocketHub
{
    /** Open an outgoing websocket
     *
     * @param str_url {string}
     **/
    imm_webSocketOpen(str_url: string): void
    {
        let websock = new WebSocket(str_url);
        websock.binaryType = 'arraybuffer';
        this.websocket = websock;
    }

    /** Fills a buffer with random numbers
     *
     * @param arr {Uint8Array}
     **/
    imm_getRandomValues(arr: Uint8Array): Uint8Array
    {
        return <Uint8Array>window.crypto.getRandomValues(arr);
    }
}